@@ -2,6 +2,8 @@
 
 import sys
 import os
+import time
+import math
 import subprocess
 import linecache
 
@@ -11,17 +13,66 @@ import click
 TOTAL_PROBLEMS = 202
 
 
+def format_time(timespan, precision=3):
+    """Formats the timespan in a human readable form"""
+
+    if timespan >= 60.0:
+        # Format time greater than one minute in a human readable format
+        # Idea from http://snipplr.com/view/5713/
+        parts = [('d', 60*60*24), ('h', 60*60), ('min', 60), ('s', 1)]
+        time_parts = []
+        leftover = timespan
+        for suffix, length in parts:
+            value = int(leftover / length)
+            if value > 0:
+                leftover = leftover % length
+                time_parts.append('%s%s' % (str(value), suffix))
+            if leftover < 1:
+                break
+        return ' '.join(time_parts)
+
+
+    # Unfortunately the unicode 'micro' symbol can cause problems in
+    # certain terminals.
+    # See bug: https://bugs.launchpad.net/ipython/+bug/348466
+    # Try to prevent crashes by being more secure than it needs to be
+    # eg. Eclipse is able to print a µ, but has no sys.stdout.encoding set.
+    units = ['s', 'ms', 'us', 'ns'] # the save value
+    if hasattr(sys.stdout, 'encoding') and sys.stdout.encoding:
+        try:
+            micro = b'\xc2\xb5s'.decode('utf-8')
+            units = ['s', 'ms', micro, 'ns']
+        except:
+            pass
+    scaling = [1, 1e3, 1e6, 1e9]
+
+    if timespan > 0.0:
+        order = min(-int(math.floor(math.log10(timespan)) // 3), 3)
+    else:
+        order = 3
+    return '%.*g %s' % (precision, timespan * scaling[order], units[order])
+
+
 def get_filename(problem):
     """Returns filename in the form `001.py`"""
     return '{0:03d}.py'.format(problem)
 
 
 def get_solution(problem):
-    """Returns the solution to a given problem"""
+    """Returns the answer to a given problem"""
     solutionsFile = os.path.join(os.path.dirname(__file__), 'solutions.txt')
     line = linecache.getline(solutionsFile, problem)
 
-    return line.split(".", 1)[1][1:-1] # strip space following "." and newline
+    # Isolate answer from the question number and trailing newline
+    answer = line.split(". ")[1].strip()
+
+    if answer == '':
+        click.echo('No known answer for problem #{0}.'.format(problem))
+        click.echo('If you have an answer, consider submitting a pull ' +
+            'request at https://github.com/iKevinY/EulerPy.')
+        return None
+    else:
+        return answer
 
 
 def verify_answer(problem):
@@ -31,37 +82,49 @@ def verify_answer(problem):
         click.secho('Error: "{0}" not found.'.format(filename), fg='red')
         sys.exit(1)
 
-    click.echo('Checking "{0}" against solution: '.format(filename), nl=False)
+    solution = get_solution(problem)
 
-    cmd = 'python {0}'.format(filename)
-    proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
-    output, _ = proc.communicate()
+    if solution:
+        click.echo('Checking "{0}" against solution: '.format(filename), nl=False)
 
-    # python3 returns bytes, use a valid encoding like ascii as our output will
-    # fall in that range
-    if isinstance(output, bytes):
-      output = output.decode('ascii')
+        cmd = 'python {0}'.format(filename)
+        wall_start = time.time()
+        proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
+        output, _ = proc.communicate()
+        wall_end = time.time()
 
-    retval = proc.poll()
-    if retval:
-        click.secho('Error calling "{0}".'.format(filename), fg='red')
-        sys.exit(1)
+        # Calculate the wall time and format the output
+        wall_time = wall_end - wall_start
+        time_info = ' (time elapsed: {0})'.format(format_time(wall_time))
 
-    # Strip newline from end of output if output is not a lone newline.
-    # This behaviour is favourable to stripping all whitespace with strip()
-    # as stripping all newlines from the output may inhib debugging done by
-    # the user (if they were to use multiple print statements in their code
-    # while in the process of atempting to solve the problem).
-    try:
-        if output[-1] == '\n':
-            output = output[:-1]
-    except IndexError:
-        output = "[no output]"
+        # Python 3 returns bytes; use a valid encoding like ASCII as the output
+        # will fall in that range
+        if isinstance(output, bytes):
+            output = output.decode('ascii')
 
-    isCorrect = output == get_solution(problem)
-    click.secho(output, bold=True, fg=('green' if isCorrect else 'red'))
+        return_val = proc.poll()
 
-    return isCorrect
+        if return_val:
+            click.secho('Error calling "{0}".'.format(filename), fg='red')
+            sys.exit(1)
+
+        # Strip newline from end of output if output is not a lone newline.
+        # This behaviour is favourable to stripping all whitespace with strip()
+        # as stripping all newlines from the output may inhib debugging done by
+        # the user (if they were to use multiple print statements in their code
+        # while in the process of atempting to solve the problem).
+        try:
+            if output[-1] == '\n':
+                output = output[:-1]
+        except IndexError:
+            output = "[no output]"
+
+        is_correct = output == solution
+        click.secho(
+            output, bold=True, nl=False, fg=('green' if is_correct else 'red')
+        )
+        click.secho(time_info, fg='cyan')
+        return is_correct
 
 
 def get_problem(problem):
@@ -91,13 +154,15 @@ def get_problem(problem):
 
 
 def generate_file(problem, default=True):
-    click.confirm("Generate file for problem #{0}?".format(problem), default=default, abort=True)
+    click.confirm("Generate file for problem #{0}?".format(problem),
+                  default=default, abort=True)
     problemText = get_problem(problem)
 
     filename = get_filename(problem)
 
     if os.path.isfile(filename):
-        click.secho('"{0}" already exists. Overwrite?'.format(filename), fg='red', nl=False)
+        click.secho('"{0}" already exists. Overwrite?'.format(filename),
+                    fg='red', nl=False)
         click.confirm('', abort=True)
 
     problemHeader = 'Project Euler Problem #{0}\n'.format(problem)
@@ -119,15 +184,18 @@ def generate_first_problem():
 
 
 def view_solution(problem):
-    click.confirm("View answer to problem #{0}?".format(problem), abort=True)
-    click.echo("The answer to problem #{0} is ".format(problem), nl=False)
-    click.secho(get_solution(problem), bold=True, nl=False)
-    click.echo(".")
+    solution = get_solution(problem)
+
+    if solution:
+        click.confirm("View answer to problem #{0}?".format(problem), abort=True)
+        click.echo("The answer to problem #{0} is ".format(problem), nl=False)
+        click.secho(solution, bold=True, nl=False)
+        click.echo(".")
 
 
 def preview_problem(problem):
     click.secho("Project Euler Problem #{0}".format(problem), bold=True)
-    click.echo(get_problem(problem)[:-1]) # strip trailing newline
+    click.echo(get_problem(problem)[:-1])  # strip trailing newline
 
 
 def determine_largest_problem():
@@ -139,13 +207,14 @@ def determine_largest_problem():
 
 
 help = {
-    'cheat': "View the answer to a problem.",
-    'generate': "Generates Python file for a problem.",
-    'skip': "Generates Python file for the next problem.",
-    'preview': "Prints the text of a problem.",
-    'verify': "Verifies the solution to a problem.",
+    'cheat': 'View the answer to a problem.',
+    'generate': 'Generates Python file for a problem.',
+    'skip': 'Generates Python file for the next problem.',
+    'preview': 'Prints the text of a problem.',
+    'verify': 'Verifies the solution to a problem.',
 }
 
+
 @click.command(name='EulerPy')
 @click.argument('problem', default=0, type=click.IntRange(0, TOTAL_PROBLEMS))
 @click.option('--cheat',    '-c', 'option', flag_value='cheat', help=help['cheat'])
